@@ -238,10 +238,12 @@ static void init_colors (void)
 
    fg = "black";
    SLtt_set_color (1, NULL, fg, "default");
+   SLtt_set_color (1+NUM_COLORS, NULL, "default", "fg");
    for (i = 1; i < NUM_COLORS; i++)
      {
 	bg = Colors[i];
 	SLtt_set_color (i + 1, NULL, fg, bg);
+	SLtt_set_color (i + 1 + NUM_COLORS, NULL, bg, fg);
      }
 }
 
@@ -349,7 +351,7 @@ static void color_test (void)
 	color = color % NUM_COLORS;
 
 	SLsmg_gotorc (row, 0);
-	SLsmg_set_color (0);
+	SLsmg_set_color (color+1+NUM_COLORS);
 	SLsmg_write_string (Colors[color]);
 	color++;
 	SLsmg_set_color (color);
