@@ -588,6 +588,8 @@ extern int _pSLpush_alloced_slstring (char *, unsigned int);
 
 
 extern unsigned int _pSLstring_bytelen (SLstr_Type *);
+extern void _pSLang_free_slstring (SLstr_Type *);   /* slstring required and assumed */
+extern unsigned long _pSLstring_get_hash (SLstr_Type *s);   /* slstring required */
 
 typedef struct 
 {
@@ -1390,5 +1392,15 @@ extern int _pSLang_check_signals_hook (VOID_STAR);
 # define _INLINE_
 #endif
 
+/* This is a macro that permits:
+ *
+ *   extern fun (void **addr);
+ *   double x;
+ *   fun ((void **) &x);   <--- warning
+ *   fun (VOID_STAR_STAR(&x)); 
+ * 
+ * Otherwise a type-punning warning could result due to strict-aliasing.
+ */
+#define VOID_STAR_STAR(addr) (VOID_STAR *)(VOID_STAR)(addr)
 
 #endif				       /* PRIVATE_SLANG_H_ */
