@@ -22,8 +22,8 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
 USA.  
 */
 
-#define SLANG_VERSION 20004
-#define SLANG_VERSION_STRING "2.0.4"
+#define SLANG_VERSION 20005
+#define SLANG_VERSION_STRING "2.0.5"
 /* #ifdef __DATE__ */
 /* # define SLANG_VERSION_STRING SLANG_VERSION_STRING0 " " __DATE__ */
 /* #else */
@@ -711,8 +711,9 @@ SL_EXTERN int SLclass_set_acopy_function (SLang_Class_Type *, int (*)(SLtype, VO
 SL_EXTERN int SLclass_set_deref_function (SLang_Class_Type *, int (*)(SLtype, VOID_STAR));
 SL_EXTERN int SLclass_set_eqs_function (SLang_Class_Type *, int (*)(SLtype, VOID_STAR, SLtype, VOID_STAR));
 
-SL_EXTERN int SLclass_set_is_container (SLang_Class_Type *, int);
+SL_EXTERN int SLclass_set_length_function (SLang_Class_Type *, int(*)(SLtype, VOID_STAR, unsigned int *));
 
+SL_EXTERN int SLclass_set_is_container (SLang_Class_Type *, int);
 SL_EXTERN int SLclass_set_foreach_functions (
   SLang_Class_Type *,
   SLang_Foreach_Context_Type *(*)(SLtype, unsigned int),   /* open method */
@@ -1291,6 +1292,13 @@ SL_EXTERN unsigned long long SLatoull (unsigned char *s);
 SL_EXTERN int SLang_pop_fileptr (SLang_MMT_Type **, FILE **);
 SL_EXTERN char *SLang_get_name_from_fileptr (SLang_MMT_Type *);
 
+SL_EXTERN int SLang_get_fileptr (SLang_MMT_Type *, FILE **);
+/* This function may be used to obtain the FILE* object associated with an MMT.
+ * It returns 0 if no-errors were encountered, and -1 otherwise.  
+ * If FILE* object has been closed, this function will return 0 and set the FILE* 
+ * parameter to NULL.
+ */
+
 typedef struct _pSLFile_FD_Type SLFile_FD_Type;
 SL_EXTERN SLFile_FD_Type *SLfile_create_fd (char *, int);
 SL_EXTERN void SLfile_free_fd (SLFile_FD_Type *);
