@@ -1,5 +1,5 @@
 /*
-Copyright (C) 2004, 2005, 2006, 2007 John E. Davis
+Copyright (C) 2004, 2005 John E. Davis
 
 This file is part of the S-Lang Library.
 
@@ -134,7 +134,7 @@ SLtype _pSLarith_Arith_Types [MAX_ARITHMETIC_TYPES+1] =
 static SLtype Alias_Map [MAX_ARITHMETIC_TYPES];
    
 /* Here are a bunch of functions to convert from one type to another.  To
- * facilitate the process, macros will be used.
+ * facilitate the process, a macros will be used.
  */
 
 #define DEFUN_1(f,from_type,to_type) \
@@ -375,8 +375,6 @@ SLarith_get_to_double_fun (SLtype type, unsigned int *sizeof_type)
 
 #define GENERIC_UNARY_FUNCTION char_unary_op
 #define GENERIC_ARITH_UNARY_FUNCTION char_arith_unary_op
-#define GENERIC_BINARY_FUNCTION char_char_arith_bin_op
-#define JUST_BOOLEAN_BINARY_OPS
 #define GENERIC_BIT_OPERATIONS
 #define GENERIC_TYPE signed char
 #define ABS_FUNCTION abs
@@ -548,20 +546,8 @@ static int arith_bin_op (int op,
    SLtype c_type;
    int ret;
 
-   if ((a_type == b_type)
-       && ((a_type == SLANG_CHAR_TYPE) || (a_type == SLANG_UCHAR_TYPE)))
-     {
-	switch (op)
-	  {
-	   case SLANG_EQ:
-	   case SLANG_NE:
-	   case SLANG_AND:
-	   case SLANG_OR:
-	     return char_char_arith_bin_op (op, a_type, ap, na, b_type, bp, nb, cp);
-	  }
-     }
-
    c_type = promote_to_common_type (a_type, b_type);
+
    a_indx = TYPE_TO_TABLE_INDEX(a_type);
    b_indx = TYPE_TO_TABLE_INDEX(b_type);
    c_indx = TYPE_TO_TABLE_INDEX(c_type);
