@@ -60,7 +60,7 @@ static int pop_array (SLang_Array_Type **at_ptr, int convert_scalar)
 	return SLclass_pop_ptr_obj (SLANG_ARRAY_TYPE, (VOID_STAR *) at_ptr);
 
       case SLANG_NULL_TYPE:
-	convert_scalar = 0;
+	/* convert_scalar = 0; */  /* commented out for 2.0.5 to fix array_map NULL bug */
 	/* drop */
       default:
 	if (convert_scalar == 0)
@@ -1253,8 +1253,8 @@ int _pSLarray_aget1 (unsigned int num_indices)
 	     if (-1 == SLang_pop_slstring (&s))
 	       return -1;
 
-	     ret = push_string_element (type, (unsigned char *)s, strlen (s));
-	     SLang_free_slstring (s);
+	     ret = push_string_element (type, (unsigned char *)s, _pSLstring_bytelen (s));
+	     _pSLang_free_slstring (s);
 	     return ret;
 	  }
 	break;
@@ -1863,7 +1863,7 @@ static void init_char_array (void)
 	goto free_and_return;
      }
 
-   n = strlen (s);
+   n = _pSLstring_bytelen (s);
    ndim = at->num_elements;
    if (n > ndim)
      {
@@ -1876,7 +1876,7 @@ static void init_char_array (void)
 
    free_and_return:
    SLang_free_array (at);
-   SLang_free_slstring (s);
+   _pSLang_free_slstring (s);
 }
 
 
@@ -2752,42 +2752,141 @@ static void is_null_intrinsic (SLang_Array_Type *at)
    SLang_push_array (bt, 1);
 }
    
+static SLang_Array_Type *pop_bool_array (void)
+{
+   SLang_Array_Type *at;
+   SLang_Array_Type *tmp_at;
+   int zero;
+
+   if (-1 == SLang_pop_array (&at, 1))
+     return NULL;
+
+   if (at->data_type == SLANG_CHAR_TYPE)
+     return at;
+
+   tmp_at = at;
+   zero = 0;
+   if (1 != array_binary_op (SLANG_NE,
+			     SLANG_ARRAY_TYPE, (VOID_STAR) &at, 1,
+			     SLANG_CHAR_TYPE, (VOID_STAR) &zero, 1,
+			     (VOID_STAR) &tmp_at))
+     {
+	SLang_free_array (at);
+	return NULL;
+     }
+
+   SLang_free_array (at);
+   at = tmp_at;
+   if (at->data_type != SLANG_CHAR_TYPE)
+     {
+	SLang_free_array (at);
+	SLang_set_error (SL_TYPE_MISMATCH);
+	return NULL;
+     }
+   return at;
+}
+
+static int pop_bool_array_and_start (int nargs, SLang_Array_Type **atp, SLindex_Type *sp)
+{
+   SLang_Array_Type *at;
+   SLindex_Type istart = *sp;
+   SLindex_Type num_elements;
+
+   if (nargs == 2)
+     {
+	if (-1 == SLang_pop_int (&istart))
+	  return -1;
+     }
+
+   if (NULL == (at = pop_bool_array ()))
+     return -1;
+
+   num_elements = (SLindex_Type) at->num_elements;
+
+   if (istart < 0)
+     istart += num_elements;
+
+   if (istart >= num_elements)
+     {
+	SLang_set_error (SL_Index_Error);
+	SLang_free_array (at);
+	return -1;
+     }
    
+   *atp = at;
+   *sp = istart;
+   return 0;
+}
+
+/* Usage: i = wherefirst (at [,startpos]); */
+static void array_where_first (void)
+{
+   SLang_Array_Type *at;
+   char *a_data;
+   SLindex_Type i, num_elements;
+   SLindex_Type istart = 0;
+   
+   istart = 0;
+   if (-1 == pop_bool_array_and_start (SLang_Num_Function_Args, &at, &istart))
+     return;
+
+   a_data = (char *) at->data;
+   num_elements = (SLindex_Type) at->num_elements;
+
+   for (i = istart; i < num_elements; i++)
+     {
+	if (a_data[i] == 0)
+	  continue;
+	
+	(void) SLang_push_int (i);
+	SLang_free_array (at);
+	return;
+     }
+   SLang_free_array (at);
+   SLang_push_null ();
+}
+
+/* Usage: i = wherelast (at [,startpos]); */
+static void array_where_last (void)
+{
+   SLang_Array_Type *at;
+   char *a_data;
+   SLindex_Type i, num_elements;
+   SLindex_Type istart = 0;
+   
+   istart = -1;
+   if (-1 == pop_bool_array_and_start (SLang_Num_Function_Args, &at, &istart))
+     return;
+
+   a_data = (char *) at->data;
+   num_elements = (SLindex_Type) at->num_elements;
+
+   i = istart + 1;
+   while (i > 0)
+     {
+	i--;
+	if (a_data[i] == 0)
+	  continue;
+	
+	(void) SLang_push_int (i);
+	SLang_free_array (at);
+	return;
+     }   
+   SLang_free_array (at);
+   SLang_push_null ();
+}
+
 static void array_where (void)
 {
    SLang_Array_Type *at, *bt;
    char *a_data;
    SLindex_Type *b_data;
-   unsigned int i, num_elements;
+   SLuindex_Type i, num_elements;
    SLindex_Type b_num;
 
-   if (-1 == SLang_pop_array (&at, 1))
+   if (NULL == (at = pop_bool_array ()))
      return;
 
-   bt = NULL;
-
-   if (at->data_type != SLANG_CHAR_TYPE)
-     {
-	int zero;
-	SLang_Array_Type *tmp_at;
-
-	tmp_at = at;
-	zero = 0;
-	if (1 != array_binary_op (SLANG_NE,
-				  SLANG_ARRAY_TYPE, (VOID_STAR) &at, 1,
-				  SLANG_CHAR_TYPE, (VOID_STAR) &zero, 1,
-				  (VOID_STAR) &tmp_at))
-	    goto return_error;
-
-	SLang_free_array (at);
-	at = tmp_at;
-	if (at->data_type != SLANG_CHAR_TYPE)
-	  {
-	     SLang_set_error (SL_TYPE_MISMATCH);
-	     goto return_error;
-	  }
-     }
-
    a_data = (char *) at->data;
    num_elements = at->num_elements;
 
@@ -3276,6 +3375,8 @@ static SLang_Intrin_Fun_Type Array_Table [] =
    MAKE_INTRINSIC_0("array_info", array_info, SLANG_VOID_TYPE),
    MAKE_INTRINSIC_0("array_shape", array_shape, SLANG_VOID_TYPE),
    MAKE_INTRINSIC_0("where", array_where, SLANG_VOID_TYPE),
+   MAKE_INTRINSIC_0("wherefirst", array_where_first, SLANG_VOID_TYPE),
+   MAKE_INTRINSIC_0("wherelast", array_where_last, SLANG_VOID_TYPE),
    MAKE_INTRINSIC_0("reshape", array_reshape, SLANG_VOID_TYPE),
    MAKE_INTRINSIC_0("_reshape", _array_reshape, SLANG_VOID_TYPE),
 #if 0
