@@ -263,12 +263,12 @@ static SLang_Assoc_Array_Type *alloc_assoc_array (SLtype type, int has_default_v
 }
 
 
-static int store_object (SLang_Assoc_Array_Type *a, char *s, SLang_Object_Type *obj)
+static int store_object (SLang_Assoc_Array_Type *a, SLstr_Type *s, SLang_Object_Type *obj)
 {
    unsigned long hash;
    _pSLAssoc_Array_Element_Type *e;
 
-   hash = _pSLcompute_string_hash (s);
+   hash = _pSLstring_get_hash (s);
    if (NULL != (e = find_element (a, s, hash)))
      SLang_free_object (&e->value);
    else
@@ -302,7 +302,7 @@ static void assoc_destroy (SLtype type, VOID_STAR ptr)
 static int pop_index (unsigned int num_indices,
 		      SLang_MMT_Type **mmt,
 		      SLang_Assoc_Array_Type **a,
-		      char **str)
+		      SLstr_Type **str)
 {
    if (NULL == (*mmt = SLang_pop_mmt (SLANG_ASSOC_TYPE)))
      {
@@ -330,9 +330,8 @@ static int pop_index (unsigned int num_indices,
 int _pSLassoc_aget (SLtype type, unsigned int num_indices)
 {
    unsigned long hash;
-   unsigned int len;
    SLang_MMT_Type *mmt;
-   char *str;
+   SLstr_Type *str;
    _pSLAssoc_Array_Element_Type *e;
    SLang_Assoc_Array_Type *a;
    SLang_Object_Type *obj;
@@ -343,8 +342,7 @@ int _pSLassoc_aget (SLtype type, unsigned int num_indices)
    if (-1 == pop_index (num_indices, &mmt, &a, &str))
      return -1;
 
-   len = strlen (str);
-   hash = _pSLstring_hash ((unsigned char *)str, (unsigned char *)str + len);
+   hash = _pSLstring_get_hash (str);
    e = find_element (a, str, hash);
 
    if (e == NULL)
@@ -371,7 +369,7 @@ int _pSLassoc_aget (SLtype type, unsigned int num_indices)
    
    free_and_return:
 
-   _pSLfree_hashed_string (str, len, hash);
+   _pSLang_free_slstring (str);
    SLang_free_mmt (mmt);
    return ret;
 }
@@ -379,7 +377,7 @@ int _pSLassoc_aget (SLtype type, unsigned int num_indices)
 int _pSLassoc_aput (SLtype type, unsigned int num_indices)
 {
    SLang_MMT_Type *mmt;
-   char *str;
+   SLstr_Type *str;
    SLang_Assoc_Array_Type *a;
    SLang_Object_Type obj;
    int ret;
@@ -412,7 +410,7 @@ int _pSLassoc_aput (SLtype type, unsigned int num_indices)
      }
 
    the_return:
-   SLang_free_slstring (str);
+   _pSLang_free_slstring (str);
    SLang_free_mmt (mmt);
    return ret;
 }
@@ -587,11 +585,11 @@ static void assoc_delete_key (SLang_Assoc_Array_Type *a, char *key)
 {
    _pSLAssoc_Array_Element_Type *e;
 
-   e = find_element (a, key, _pSLcompute_string_hash (key));
+   e = find_element (a, key, _pSLstring_get_hash (key));
    if (e == NULL)
      return;
    
-   SLang_free_slstring (e->key);
+   _pSLang_free_slstring (e->key);
    SLang_free_object (&e->value);
    e->key = Deleted_Key;
    a->num_deleted++;
@@ -667,12 +665,12 @@ cl_foreach_open (SLtype type, unsigned int num)
 	     SLang_verror (SL_NOT_IMPLEMENTED,
 			   "using '%s' not supported by SLassoc_Type",
 			   s);
-	     SLang_free_slstring (s);
+	     _pSLang_free_slstring (s);
 	     SLang_free_mmt (mmt);
 	     return NULL;
 	  }
 
-	SLang_free_slstring (s);
+	_pSLang_free_slstring (s);
      }
 
    if (NULL == (c = (SLang_Foreach_Context_Type *) SLmalloc (sizeof (SLang_Foreach_Context_Type))))
