@@ -157,7 +157,7 @@ void SLbstring_free (SLang_BString_Type *b)
 	break;
 
       case IS_SLSTRING:
-	SLang_free_slstring ((char *)b->v.ptr);
+	_pSLang_free_slstring ((char *)b->v.ptr);
 	break;
 
       case IS_MALLOCED:
@@ -525,7 +525,7 @@ static int bstring_to_string (SLtype a_type, VOID_STAR ap, unsigned int na,
 	     while (i != 0)
 	       {
 		  i--;
-		  SLang_free_slstring (s[i]);
+		  _pSLang_free_slstring (s[i]);
 		  s[i] = NULL;
 	       }
 	     return -1;
