@@ -89,6 +89,7 @@ static SLang_Class_Type **find_empty_class_slot (SLtype *typep, Class_Table_Type
    Class_Table_Type *t;
    SLtype type;
 
+   /* Class_Tables[0] is reserved (0 <= SLtype < 256) by interpreter */
    for (i = 1; i < 256; i++)
      {
 	unsigned int j;
@@ -1129,6 +1130,14 @@ int SLclass_set_eqs_function (SLang_Class_Type *cl, int (*f)(SLtype, VOID_STAR,
    return 0;
 }
 
+int SLclass_set_length_function (SLang_Class_Type *cl, int (*f)(SLtype, VOID_STAR, unsigned int *))
+{
+   if (cl == NULL) return -1;
+   cl->cl_length = f;
+
+   return 0;
+}
+
 extern int SLclass_set_is_container (SLang_Class_Type *cl, int ic)
 {
    if (cl == NULL)
@@ -1566,7 +1575,7 @@ SLang_MMT_Type *SLang_pop_mmt (SLtype type) /*{{{*/
 {
    SLang_MMT_Type *mmt;
 
-   if (-1 == SLclass_pop_ptr_obj (type, (VOID_STAR *) &mmt))
+   if (-1 == SLclass_pop_ptr_obj (type, VOID_STAR_STAR(&mmt)))
      mmt = NULL;
    return mmt;
 
