@@ -44,7 +44,7 @@ int SLang_pop_complex (double *r, double *i)
    switch (SLang_peek_at_stack ())
      {
       case SLANG_COMPLEX_TYPE:
-	if (-1 == SLclass_pop_ptr_obj (SLANG_COMPLEX_TYPE, (VOID_STAR *)&c))
+	if (-1 == SLclass_pop_ptr_obj (SLANG_COMPLEX_TYPE, VOID_STAR_STAR(&c)))
 	  return -1;
 	*r = c[0];
 	*i = c[1];
