@@ -1,12 +1,17 @@
 #ifndef _SLANG_INCLUDE_H_
 #define _SLANG_INCLUDE_H_
 
+#define SLANG_SOURCE_	1
 #include "config.h"
 #include "sl-feat.h"
 
 #include <stdio.h>
 #include <string.h>
 
+#if defined(__QNX__) && defined(__WATCOMC__)
+# include <unix.h>
+#endif
+  
 #ifdef HAVE_STDLIB_H
 # include <stdlib.h>
 #endif
