@@ -42,7 +42,7 @@ static SLang_IStruct_Field_Type *istruct_pop_field (char *name, int no_readonly,
    char *struct_addr;
 
    /* Note: There is no need to free this object */
-   if (-1 == SLclass_pop_ptr_obj (SLANG_ISTRUCT_TYPE, (VOID_STAR *) &s))
+   if (-1 == SLclass_pop_ptr_obj (SLANG_ISTRUCT_TYPE, VOID_STAR_STAR(&s)))
      return NULL;
 
    if (NULL == (struct_addr = *(char **)s->addr))
