@@ -953,6 +953,105 @@ static void diff_fun (void)
 {
    (void) do_binary_function (do_diff);
 }
+
+static int do_nint (double x)
+{
+   double xf, xi;
+   
+   xf = modf (x, &xi);		       /* x = xi + xf */
+   if (xi > 0)
+     {
+	if (xf >= 0.5)
+	  return xi + 1;
+     }
+   else 
+     {
+	if (xf <= -0.5)
+	  return xi - 1;
+     }
+   return xi;
+}
+
+static int float_to_nint (SLang_Array_Type *at, SLang_Array_Type *bt)
+{
+   unsigned int n, i;
+   int *ip;
+   float *fp;
+   
+   fp = (float *) at->data;
+   ip = (int *) bt;
+   n = at->num_elements;
+
+   for (i = 0; i < n; i++)
+     ip[i] = do_nint ((double) fp[i]);
+
+   return 0;
+}
+
+static int double_to_nint (SLang_Array_Type *at, SLang_Array_Type *bt)
+{
+   unsigned int n, i;
+   int *ip;
+   double *dp;
+   
+   dp = (double *) at->data;
+   ip = (int *) bt;
+   n = at->num_elements;
+
+   for (i = 0; i < n; i++)
+     ip[i] = do_nint (dp[i]);
+
+   return 0;
+}
+
+static void nint_intrin (void)
+{
+   double x;
+   SLang_Array_Type *at, *bt;
+   int (*at_to_int_fun)(SLang_Array_Type *, SLang_Array_Type *);
+
+   if (SLang_peek_at_stack () != SLANG_ARRAY_TYPE)
+     {
+	if (-1 == SLang_pop_double (&x))
+	  return;
+	(void) SLang_push_int (do_nint (x));
+	return;
+     }
+   switch (SLang_peek_at_stack1 ())
+     {
+      case -1:
+	return;
+
+      case SLANG_INT_TYPE:
+	return;
+
+      case SLANG_FLOAT_TYPE:
+	if (-1 == SLang_pop_array_of_type (&at, SLANG_FLOAT_TYPE))
+	  return;
+	at_to_int_fun = float_to_nint;
+	break;
+
+      case SLANG_DOUBLE_TYPE:
+      default:
+	if (-1 == SLang_pop_array_of_type (&at, SLANG_FLOAT_TYPE))
+	  return;
+	at_to_int_fun = double_to_nint;
+	break;
+     }
+   
+   if (NULL == (bt = SLang_create_array1 (SLANG_INT_TYPE, 0, NULL, at->dims, at->num_dims, 1)))
+     {
+	SLang_free_array (at);
+	return;
+     }
+   if (0 == (*at_to_int_fun) (at, bt))
+     (void) SLang_push_array (bt, 0);
+   
+   SLang_free_array (bt);
+   SLang_free_array (at);
+}
+
+
 static void fpu_clear_except_bits (void)
 {
    SLfpu_clear_except_bits ();
@@ -1004,6 +1103,7 @@ static SLang_Math_Unary_Type SLmath_Table [] =
 
 static SLang_Intrin_Fun_Type SLang_Math_Table [] =
 {
+   MAKE_INTRINSIC_0("nint", nint_intrin, SLANG_VOID_TYPE),
    MAKE_INTRINSIC_0("polynom", math_poly, SLANG_DOUBLE_TYPE),
    MAKE_INTRINSIC_0("hypot", hypot_fun, SLANG_VOID_TYPE),
    MAKE_INTRINSIC_0("atan2", atan2_fun, SLANG_VOID_TYPE),
