@@ -259,6 +259,23 @@ static int append_token_of_type (unsigned char t)
    /* The memset when the list was created ensures that the other fields
     * are properly initialized.
     */
+   if ((t == CHS_TOKEN) && Token_List->len)
+     {
+	tok = Token_List->stack + (Token_List->len-1);
+	if (IS_INTEGER_TOKEN(tok->type))
+	  {
+	     tok->v.long_val = -tok->v.long_val;
+	     return 0;
+	  }
+#ifdef HAVE_LONG_LONG
+	if ((tok->type == LLONG_TOKEN) || (tok->type == ULLONG_TOKEN))
+	  {
+	     tok->v.llong_val = -tok->v.llong_val;
+	     return 0;
+	  }
+#endif
+     }
+
    tok = Token_List->stack + Token_List->len;
    init_token (tok);
    tok->type = t;
@@ -1861,6 +1878,7 @@ static void unary_expression (_pSLang_Token_Type *ctok)
 
 	   case SUB_TOKEN:
 	     (void) get_token (ctok);
+#if 0
 	     if (IS_INTEGER_TOKEN (ctok->type))
 	       {
 		  ctok->v.long_val = -ctok->v.long_val;
@@ -1873,7 +1891,7 @@ static void unary_expression (_pSLang_Token_Type *ctok)
 		  break;
 	       }
 #endif
-
+#endif
 	     if (num_unary_ops == 16)
 	       goto stack_overflow_error;
 	     save_unary_ops [num_unary_ops++] = CHS_TOKEN;
@@ -1906,28 +1924,6 @@ static void unary_expression (_pSLang_Token_Type *ctok)
      }
 
    out_of_switch:
-   if (num_unary_ops == 0)
-     return;
-#if 0
-   if ((DEREF_TOKEN == save_unary_ops[num_unary_ops - 1])
-       && (NULL != (last_token = get_last_token ()))
-       && (IS_ASSIGN_TOKEN(last_token->type)))
-     {
-	/* FIXME: Priority=medium
-	 * This needs generalized so that things like @a.y = 1 will work properly.
-	 */
-	if ((num_unary_ops != 1)
-	    || (last_token->type != _SCALAR_ASSIGN_TOKEN))
-	  {
-	     SLang_verror (SL_NOT_IMPLEMENTED, 
-			   "Only derefence assignments to simple variables are possible");
-	     return;
-	  }
-
-	last_token->type += (_DEREF_ASSIGN_TOKEN - _SCALAR_ASSIGN_TOKEN);
-	return;
-     }
-#endif
    while (num_unary_ops)
      {
 	num_unary_ops--;
