@@ -113,6 +113,25 @@ USA.
 #include "slang.h"
 #include "_slang.h"
 
+static int is_interrupt (int e)
+{
+#ifdef EINTR
+   if (e == EINTR)
+     {
+	if (0 == SLang_handle_interrupt ())
+	  return 1;
+     }
+#endif
+#ifdef EAGAIN
+   if (e == EAGAIN)
+     {
+	if (0 == SLang_handle_interrupt ())
+	  return 1;
+     }
+#endif
+   return 0;
+}
+
 typedef struct
 {
    struct stat st;
@@ -151,38 +170,44 @@ static void stat_cmd (char *file)
    struct stat st;
    int status;
    int opt_attrs;
-
-   status = stat (file, &st);
-
 #if defined(__MSDOS__) || defined(__WIN32__)
-   if (status == -1)
-     {
-	unsigned int len = strlen (file);
-	if (len && ((file[len-1] == '\\') || (file[len-1] == '/')))
-	  {
-	     file = SLmake_nstring (file, len-1);
-	     if (file == NULL)
-	       return;
+   unsigned int len = strlen (file);
+   int is_malloced = 0;
 
-	     status = stat (file, &st);
-	     SLfree (file);
-	  }
+   if (len && ((file[len-1] == '\\') || (file[len-1] == '/')))
+     {
+	file = SLmake_nstring (file, len-1);
+	if (file == NULL)
+	  return;
+	
+	is_malloced = 1;
      }
 #endif
-   if (status == -1)
+
+   while ((-1 == (status = stat (file, &st)))
+	  && (is_interrupt (errno)))
+     continue;
+
+   if (status == 0)
+     {
+#ifdef __WIN32__
+	opt_attrs = GetFileAttributes (file);
+#else
+	opt_attrs = 0;
+#endif
+
+	push_stat_struct (&st, opt_attrs);
+     }
+   else 
      {
 	_pSLerrno_errno = errno;
 	SLang_push_null ();
-	return;
      }
-
-#ifdef __WIN32__
-   opt_attrs = GetFileAttributes (file);
-#else
-   opt_attrs = 0;
+   
+#if defined(__MSDOS__) || defined(__WIN32__)
+   if (is_malloced)
+     SLfree (file);
 #endif
-
-   push_stat_struct (&st, opt_attrs);
 }
 
 static void lstat_cmd (char *file)
@@ -191,8 +216,11 @@ static void lstat_cmd (char *file)
    struct stat st;
    int opt_attrs;
 
-   if (-1 == lstat (file, &st))
+   while (-1 == lstat (file, &st))
      {
+	if (is_interrupt (errno))
+	  continue;
+
 	_pSLerrno_errno = errno;
 	SLang_push_null ();
 	return;
@@ -304,7 +332,10 @@ static void readlink_cmd (char *s)
    char buf[2048];
    int n;
 
-   n = readlink (s, buf, sizeof (buf)-1);
+   while ((-1 == (n = readlink (s, buf, sizeof (buf)-1)))
+	  && is_interrupt (errno))
+     ;
+
    if (n == -1)
      {
 	_pSLerrno_errno = errno;
@@ -324,10 +355,15 @@ static void readlink_cmd (char *s)
 static int symlink_cmd (char *oldpath, char *newpath)
 {
    int s;
-   
-   if (-1 == (s = symlink (oldpath, newpath)))
-     _pSLerrno_errno = errno;
-   
+
+   while (-1 == (s = symlink (oldpath, newpath)))
+     {
+	if (is_interrupt (errno))
+	  continue;
+
+	_pSLerrno_errno = errno;
+	break;
+     }
    return s;
 }
 #endif
@@ -337,17 +373,25 @@ static int hardlink_cmd (char *oldpath, char *newpath)
 {
    int s;
    
-   if (-1 == (s = link (oldpath, newpath)))
-     _pSLerrno_errno = errno;
-   
+   while (-1 == (s = link (oldpath, newpath)))
+     {
+	if (is_interrupt (errno))
+	  continue;
+
+	_pSLerrno_errno = errno;
+	break;
+     }
    return s;
 }
 #endif
 
 static int chmod_cmd (char *file, int *mode)
 {
-   if (-1 == chmod(file, (mode_t) *mode))
+   while (-1 == chmod(file, (mode_t) *mode))
      {
+	if (is_interrupt (errno))
+	  continue;
+
 	_pSLerrno_errno = errno;
 	return -1;
      }
@@ -357,11 +401,15 @@ static int chmod_cmd (char *file, int *mode)
 #ifdef HAVE_CHOWN
 static int chown_cmd (char *file, int *owner, int *group)
 {
-   int ret;
+   while (-1 == chown(file, (uid_t) *owner, (gid_t) *group))
+     {
+	if (is_interrupt (errno))
+	  continue;
 
-   if (-1 == (ret = chown(file, (uid_t) *owner, (gid_t) *group)))
-     _pSLerrno_errno = errno;
-   return ret;
+	_pSLerrno_errno = errno;
+	return -1;
+     }
+   return 0;
 }
 #endif
 
@@ -431,10 +479,9 @@ static int chdir_cmd (char *s)
 
    while (-1 == (ret = chdir (s)))
      {
-#ifdef EINTR
-	if (errno == EINTR)
+	if (is_interrupt (errno))
 	  continue;
-#endif
+
 	_pSLerrno_errno = errno;
 	break;
      }
@@ -539,10 +586,9 @@ static int rmdir_cmd (char *s)
 
    while (-1 == (ret = rmdir (s)))
      {
-#ifdef EINTR
-	if (errno == EINTR)
+	if (is_interrupt (errno))
 	  continue;
-#endif
+
 	_pSLerrno_errno = errno;
 	break;
      }
@@ -565,10 +611,9 @@ static int remove_cmd (char *s)
 
    while (-1 == (ret = REMOVE (s)))
      {
-#ifdef EINTR
-	if (errno == EINTR)
+	if (is_interrupt (errno))
 	  continue;
-#endif
+
 	_pSLerrno_errno = errno;
 	break;
      }
@@ -580,10 +625,9 @@ static int rename_cmd (char *oldpath, char *newpath)
    int ret;
    while (-1 == (ret = rename (oldpath, newpath)))
      {
-#ifdef EINTR
-	if (errno == EINTR)
+	if (is_interrupt (errno))
 	  continue;
-#endif
+
 	_pSLerrno_errno = errno;
 	break;
      }
@@ -622,10 +666,8 @@ static int mkdir_cmd (void)
 
    while (-1 == (ret = MKDIR(dir, mode)))
      {
-#ifdef EINTR
-	if (errno == EINTR)
+	if (is_interrupt (errno))
 	  continue;
-#endif
 	_pSLerrno_errno = errno;
 	break;
      }
@@ -637,8 +679,11 @@ static int mkdir_cmd (void)
 #ifdef HAVE_MKFIFO
 static int mkfifo_cmd (char *path, int *mode)
 {
-   if (-1 == mkfifo (path, *mode))
+   while (-1 == mkfifo (path, *mode))
      {
+	if (is_interrupt (errno))
+	  continue;
+
 	_pSLerrno_errno = errno;
 	return -1;
      }
@@ -997,6 +1042,29 @@ static int umask_cmd (int *u)
 }
 #endif
 
+#if defined(R_OK) && defined(W_OK) && defined(X_OK) && defined (F_OK)
+# define HAS_ACCESS_CMD 1
+#else
+# define HAS_ACCESS_CMD 0
+#endif
+
+#if HAS_ACCESS_CMD
+static int access_cmd (char *path, int *modep)
+{
+   int mode = *modep & (R_OK|W_OK|X_OK|F_OK);
+   
+   while (-1 == access (path, mode))
+     {
+	if (is_interrupt (errno))
+	  continue;
+	
+	_pSLerrno_errno = errno;
+	return -1;
+     }
+   return 0;
+}
+#endif
+
 static SLang_Intrin_Fun_Type PosixDir_Name_Table [] =
 {
 #ifdef HAVE_READLINK
@@ -1027,6 +1095,9 @@ static SLang_Intrin_Fun_Type PosixDir_Name_Table [] =
    MAKE_INTRINSIC_S("rmdir", rmdir_cmd, SLANG_INT_TYPE),
    MAKE_INTRINSIC_S("remove", remove_cmd, SLANG_INT_TYPE),
    MAKE_INTRINSIC_SS("rename", rename_cmd, SLANG_INT_TYPE),
+#if HAS_ACCESS_CMD
+   MAKE_INTRINSIC_SI("access", access_cmd, SLANG_INT_TYPE),
+#endif
 #if USE_LISTDIR_INTRINSIC
    MAKE_INTRINSIC("listdir", listdir_cmd_wrap, SLANG_VOID_TYPE, 0),
 #endif
@@ -1092,6 +1163,12 @@ static SLang_IConstant_Type PosixDir_Consts [] =
    MAKE_ICONSTANT("FILE_ATTRIBUTE_READONLY", FILE_ATTRIBUTE_READONLY),
    MAKE_ICONSTANT("FILE_ATTRIBUTE_SYSTEM", FILE_ATTRIBUTE_SYSTEM),
    MAKE_ICONSTANT("FILE_ATTRIBUTE_TEMPORARY", FILE_ATTRIBUTE_TEMPORARY),
+#endif
+#if HAS_ACCESS_CMD
+   MAKE_ICONSTANT("R_OK", R_OK),
+   MAKE_ICONSTANT("W_OK", W_OK),
+   MAKE_ICONSTANT("X_OK", X_OK),
+   MAKE_ICONSTANT("F_OK", F_OK),
 #endif
    SLANG_END_ICONST_TABLE
 };
