@@ -437,10 +437,13 @@ static int point_visible (int col_too)
 
 #define NEXT_CHAR_CELL \
    { \
-      if ((p->nchars != i) || (p->color != color)) flags |= TOUCHED; \
-      p->nchars = i; \
-      p->color = color; \
-      p++; \
+      if (p < pmax) \
+	{ \
+	   if ((p->nchars != i) || (p->color != color)) flags |= TOUCHED; \
+	   p->nchars = i; \
+	   p->color = color; \
+	   p++; \
+	} \
       i = 0; \
       col++; \
    } (void) 0
@@ -448,7 +451,7 @@ static int point_visible (int col_too)
 
 #define ADD_TO_CHAR_CELL(wc) \
    { \
-      if (p->wchars[i] != wc) \
+      if ((p < pmax) && (p->wchars[i] != wc)) \
 	{ \
 	   p->wchars[i] = wc; \
 	   flags |= TOUCHED; \
@@ -473,7 +476,7 @@ static int point_visible (int col_too)
 
 void SLsmg_write_chars (unsigned char *u, unsigned char *umax)
 {
-   SLsmg_Char_Type *p;
+   SLsmg_Char_Type *p, *pmax;
    SLsmg_Color_Type color;
    int flags;
    int col, start_col, max_col;
@@ -493,7 +496,7 @@ void SLsmg_write_chars (unsigned char *u, unsigned char *umax)
      display_8bit = 0xA0;
 
    color = This_Color;
-   /* If we are unicode characters for the line drawing characters, then
+   /* If we are using unicode characters for the line drawing characters, then
     * do not attempt to use the terminals alternate character set
     */
    alt_char_set_flag = (color & SLSMG_ACS_MASK);
@@ -515,10 +518,12 @@ void SLsmg_write_chars (unsigned char *u, unsigned char *umax)
    max_col = start_col + Screen_Cols;
 
    p = SL_Screen[This_Row - Start_Row].neew;
+   pmax = p + Screen_Cols;
+
    if (col >= start_col)
      {
 	p += (col - start_col);
-	if (p->nchars == 0)
+	if ((p < pmax) && (p->nchars == 0))
 	  {
 	     /* It looks like we are about to overwrite the right side of a 
 	      * double width character.
