@@ -282,6 +282,22 @@ int SLang_pop_fileptr (SLang_MMT_Type **mmt, FILE **fp)
    return 0;
 }
 
+int SLang_get_fileptr (SLang_MMT_Type *mmt, FILE **fpp)
+{
+   SL_File_Table_Type *t;
+
+   *fpp = NULL;
+   if (mmt == NULL)
+     return -1;
+
+   t = (SL_File_Table_Type *) SLang_object_from_mmt (mmt);
+   if (t == NULL)
+     return -1;
+
+   *fpp = t->fp;
+   return 0;
+}
+
 static int close_file_type (SL_File_Table_Type *t)
 {
    int ret = 0;
@@ -415,7 +431,7 @@ static int stdio_fgets (SLang_Ref_Type *ref, SL_File_Table_Type *t)
      return -1;
 
    status = SLang_assign_to_ref (ref, SLANG_STRING_TYPE, (VOID_STAR)&s);
-   SLang_free_slstring (s);
+   _pSLang_free_slstring (s);
 
    if (status == -1)
      return -1;
@@ -469,7 +485,7 @@ static void stdio_fgetslines_internal (FILE *fp, unsigned int n)
 	     new_list = (char **) SLrealloc ((char *)list, sizeof (char *) * max_num_lines);
 	     if (new_list == NULL)
 	       {
-		  SLang_free_slstring (line);
+		  _pSLang_free_slstring (line);
 		  goto return_error;
 	       }
 	     list = new_list;
@@ -576,7 +592,7 @@ static int stdio_fputs (char *s, SL_File_Table_Type *t)
      return -1;
 
    if (EOF == fputs(s, fp)) return -1;
-   return (int) strlen (s);
+   return (int) _pSLstring_bytelen (s);
 }
 
 static int stdio_fflush (SL_File_Table_Type *t)
@@ -879,17 +895,17 @@ static int stdio_fprintf (void)
    
    if (NULL == (mmt = pop_fp (SL_WRITE, &fp)))
      {
-	SLang_free_slstring (s);
+	_pSLang_free_slstring (s);
 	return -1;
      }
    
    if (EOF == fputs(s, fp))
      status = -1;
    else
-     status = (int) strlen (s);
+     status = (int) _pSLstring_bytelen (s);
 
    SLang_free_mmt (mmt);
-   SLang_free_slstring (s);
+   _pSLang_free_slstring (s);
    return status;
 }
 
@@ -907,9 +923,9 @@ static int stdio_printf (void)
    if (EOF == fputs(s, stdout))
      status = -1;
    else
-     status = (int) strlen (s);
+     status = (int) _pSLstring_bytelen (s);
 
-   SLang_free_slstring (s);
+   _pSLang_free_slstring (s);
    return status;
 }
 
@@ -1029,11 +1045,11 @@ cl_foreach_open (SLtype type, unsigned int num)
 	     SLang_verror (SL_NOT_IMPLEMENTED,
 			   "using '%s' not supported by File_Type",
 			   s);
-	     SLang_free_slstring (s);
+	     _pSLang_free_slstring (s);
 	     SLang_free_mmt (mmt);
 	     return NULL;
 	  }
-	SLang_free_slstring (s);
+	_pSLang_free_slstring (s);
 	break;
 
       default:
