@@ -102,7 +102,7 @@ static unsigned int do_trim (SLuchar_Type **beg, int do_beg,
    else white = NULL;
 
    a = *beg;
-   len = strlen ((char *)a);
+   len = _pSLstring_bytelen ((char *)a);
    b = a + len;
 
    if (do_beg)
@@ -195,7 +195,7 @@ static void strcat_cmd (void) /*{{{*/
 	if (-1 == SLang_pop_slstring (&s))
 	  goto free_and_return;
 	ptrs[i] = s;
-	len += strlen (s);
+	len += _pSLstring_bytelen (s);
      }
    if (NULL == (c = _pSLallocate_slstring (len)))
      goto free_and_return;
@@ -204,12 +204,12 @@ static void strcat_cmd (void) /*{{{*/
    for (i = 0; i < nargs; i++)
      {
 	strcpy (c1, ptrs[i]);
-	c1 += strlen (c1);
+	c1 += _pSLstring_bytelen (ptrs[i]);   /* c1 does not take advantage of the slstring cache */
      }
-   
+
    free_and_return:
    for (i = 0; i < nargs; i++)
-     SLang_free_slstring (ptrs[i]);
+     _pSLang_free_slstring (ptrs[i]);
    if (ptrs != ptrs_buf)
      SLfree ((char *) ptrs);
 
@@ -253,7 +253,7 @@ static void strtrim_cmd_internal (char *str, int do_beg, int do_end)
    
    (void) _pSLang_push_nstring ((char *)beg, len);
    if (free_str)
-     SLang_free_slstring (str);
+     _pSLang_free_slstring (str);
 }
 
    
@@ -524,7 +524,7 @@ static void strtok_cmd (char *str)
      goto the_return;
 
    s = (SLuchar_Type *)str;
-   smax = s + strlen ((char *)s);
+   smax = s + _pSLstring_bytelen ((SLstr_Type *)s);
    while (s < smax)
      {
 	SLuchar_Type *s0;
@@ -547,7 +547,7 @@ static void strtok_cmd (char *str)
 	  }
 	if (-1 == _pSLstring_list_append (&sl, new_s))
 	  {
-	     SLang_free_slstring (new_s);
+	     _pSLang_free_slstring (new_s);
 	     _pSLstring_list_delete (&sl);
 	     goto the_return;
 	  }
@@ -559,7 +559,7 @@ static void strtok_cmd (char *str)
    the_return:
    if (white != NULL)
      {
-	SLang_free_slstring (str);
+	_pSLang_free_slstring (str);
 	SLwchar_free_lut (lut);
      }
 }
@@ -782,7 +782,7 @@ static int issubstr_cmd (char *a, char *b) /*{{{*/
 
 /*}}}*/
 
-static void subbytes_cmd (char *a, int *n_ptr, int *m_ptr) /*{{{*/
+static void subbytes_cmd (SLstr_Type *a, int *n_ptr, int *m_ptr) /*{{{*/
 {
    int m;
    unsigned int n;
@@ -792,7 +792,7 @@ static void subbytes_cmd (char *a, int *n_ptr, int *m_ptr) /*{{{*/
    n = (unsigned int) (*n_ptr - 1);
    m = *m_ptr;
 
-   lena = strlen (a);
+   lena = _pSLstring_bytelen (a);
    amax = a + lena;
 
    if (n > lena) 
@@ -806,7 +806,7 @@ static void subbytes_cmd (char *a, int *n_ptr, int *m_ptr) /*{{{*/
 
 /*}}}*/
 
-static void substr_cmd (char *a, int *n_ptr, int *m_ptr) /*{{{*/
+static void substr_cmd (SLstr_Type *a, int *n_ptr, int *m_ptr) /*{{{*/
 {
    int n, m;
    int lena;
@@ -824,7 +824,7 @@ static void substr_cmd (char *a, int *n_ptr, int *m_ptr) /*{{{*/
    n = *n_ptr;
    m = *m_ptr;
 
-   lena = strlen (a);
+   lena = _pSLstring_bytelen (a);
    amax = a + lena;
 
    lena = SLutf8_strlen ((SLuchar_Type *)a, ignore_combining);
@@ -839,7 +839,8 @@ static void substr_cmd (char *a, int *n_ptr, int *m_ptr) /*{{{*/
    n--;
    if (m < 0) m = lena;
    if (n + m > lena) m = lena - n;
-   
+
+   /* FIXME: Are the strlens necessary here?  */
    a = (char *)SLutf8_skip_chars ((SLuchar_Type *)a, (SLuchar_Type*)a + strlen(a),
 				  (unsigned int)n, NULL, ignore_combining);
 
@@ -969,7 +970,7 @@ static int isdigit_cmd (void) /*{{{*/
 	  }
 	else wch = s[0];
 
-	SLang_free_slstring (s);
+	_pSLang_free_slstring (s);
      }
    else if (-1 == SLang_pop_wchar (&wch))
      return -1;
@@ -1193,20 +1194,24 @@ static int strcmp_cmd (char *a, char *b) /*{{{*/
 
 /*}}}*/
 
-static int do_strncmp_cmd (char *a, char *b, int n, int skip_combining) /*{{{*/
+static int do_strncmp_cmd (SLstr_Type *a, SLstr_Type *b, int n, int skip_combining) /*{{{*/
 {
    char *p;
    unsigned int na, nb;
+   unsigned int lena, lenb;
    int cmp;
 
    if (_pSLinterp_UTF8_Mode == 0)
      return strncmp(a, b, (unsigned int) n);
    
-   p = (char *)SLutf8_skip_chars ((SLuchar_Type *)a, (SLuchar_Type *)(a + strlen (a)),
+   lena = _pSLstring_bytelen (a);
+   lenb = _pSLstring_bytelen (b);
+
+   p = (char *)SLutf8_skip_chars ((SLuchar_Type *)a, (SLuchar_Type *)(a + lena),
                           (unsigned int) n, NULL, skip_combining);
    na = (unsigned int) (p - a);
 
-   p = (char *)SLutf8_skip_chars ((SLuchar_Type *)b, (SLuchar_Type *)(b + strlen (b)),
+   p = (char *)SLutf8_skip_chars ((SLuchar_Type *)b, (SLuchar_Type *)(b + lenb),
                           (unsigned int) n, NULL, skip_combining);
    nb = (unsigned int) (p - b);
    
@@ -1231,44 +1236,44 @@ static int do_strncmp_cmd (char *a, char *b, int n, int skip_combining) /*{{{*/
 
 /*}}}*/
 
-static int strncmp_cmd (char *a, char *b, int *n)
+static int strncmp_cmd (SLstr_Type *a, SLstr_Type *b, int *n)
 {
    return do_strncmp_cmd (a, b, *n, 1);
 }
 
-static int strnbytecmp_cmd (char *a, char *b, int *n)
+static int strnbytecmp_cmd (SLstr_Type *a, SLstr_Type *b, int *n)
 {
    return (int) strncmp (a, b, *n);
 }
 
-static int strncharcmp_cmd (char *a, char *b, int *n)
+static int strncharcmp_cmd (SLstr_Type *a, SLstr_Type *b, int *n)
 {
    return do_strncmp_cmd (a, b, *n, 0);
 }
 
-static int do_strlen_combining (char *s, int ignore_combining) /*{{{*/
+static int do_strlen_combining (SLstr_Type *s, int ignore_combining) /*{{{*/
 {
    if (_pSLinterp_UTF8_Mode == 0)
-     return (int) strlen (s);
+     return (int) _pSLstring_bytelen (s);
    
    return (int) SLutf8_strlen ((SLuchar_Type *)s, ignore_combining);
 }
 /*}}}*/
 
 
-static int strlen_cmd (char *s) /*{{{*/
+static int strlen_cmd (SLstr_Type *s) /*{{{*/
 {
    return do_strlen_combining (s, 1);
 }
 
-static int strcharlen_cmd (char *s)
+static int strcharlen_cmd (SLstr_Type *s)
 {
    return do_strlen_combining (s, 0);
 }
 
-static int strbytelen_cmd (char *s)
+static int strbytelen_cmd (SLstr_Type *s)
 {
-   return (int) strlen (s);
+   return (int) _pSLstring_bytelen (s);
 }
 
 /*}}}*/
@@ -1282,9 +1287,9 @@ static char *SLdo_sprintf (char *fmt) /*{{{*/
    char *out = NULL, *outp = NULL;
    char dfmt[1024];	       /* used to hold part of format */
    char *f;
-   VOID_STAR varp;
+   char *str;
    unsigned int want_width, width, precis;
-   int use_varp, int_var;
+   int int_var, use_string;
    long long_var;
 #ifdef HAVE_LONG_LONG
    long long llong_var;
@@ -1414,13 +1419,13 @@ static char *SLdo_sprintf (char *fmt) /*{{{*/
 #ifdef HAVE_LONG_LONG
 	llong_var = 0;
 #endif
-	varp = NULL;
+	str = NULL;
 	guess_size = 32;
 #if SLANG_HAS_FLOAT
 	use_double = 0;
 #endif
 	use_long = 0;
-	use_varp = 0;
+	use_string = 0;
 	do_free = 0;
 
 	if (ch == 'l')
@@ -1443,17 +1448,18 @@ static char *SLdo_sprintf (char *fmt) /*{{{*/
 	     ch = 's';
 	     /* drop */
 	   case 's':
-	     if (SLang_pop_slstring((char **) &varp)) return (out);
+	     if (-1 == SLang_pop_slstring(&str))
+	       return (out);
 	     do_free = 1;
-	     guess_size = strlen((char *) varp);
-	     use_varp = 1;
+	     guess_size = strlen(str);
+	     use_string = 1;
 	     break;
 
 	   case '%':
 	     guess_size = 1;
 	     do_free = 0;
-	     use_varp = 1;
-	     varp = (VOID_STAR) "%";
+	     use_string = 1;
+	     str = "%";
 	     break;
 
 	   case 'c':
@@ -1466,8 +1472,8 @@ static char *SLdo_sprintf (char *fmt) /*{{{*/
 		      || (NULL == (_pSLinterp_encode_wchar (wc, utf8_buf, &guess_size))))
 		    return out;
 		  ch = 's';
-		  varp = (char *)utf8_buf;
-		  use_varp = 1;
+		  str = (char *)utf8_buf;
+		  use_string = 1;
 		  break;
 	       }
 	     guess_size = 1;
@@ -1515,8 +1521,8 @@ static char *SLdo_sprintf (char *fmt) /*{{{*/
 	     /* Pointer type?? Why?? */
 	     if (-1 == SLdo_pop ())
 	       return out;
-	     varp = (VOID_STAR) _pSLang_get_run_stack_pointer ();
-	     use_varp = 1;
+	     str = (char *) _pSLang_get_run_stack_pointer ();
+	     use_string = 1;
 	     use_long = 0;
 	     break;
 
@@ -1542,10 +1548,10 @@ static char *SLdo_sprintf (char *fmt) /*{{{*/
 	     malloc_len = len + guess_size;
 	  }
 
-	if (use_varp)
+	if (use_string)
 	  {
-	     sprintf(outp, dfmt, varp);
-	     if (do_free) SLang_free_slstring ((char *)varp);
+	     sprintf(outp, dfmt, str);
+	     if (do_free) _pSLang_free_slstring (str);
 	  }
 #if SLANG_HAS_FLOAT
 	else if (use_double) sprintf(outp, dfmt, x);
@@ -1593,7 +1599,7 @@ int _pSLstrops_do_sprintf_n (int n) /*{{{*/
      return -1;
 
    p = SLdo_sprintf (fmt);
-   SLang_free_slstring (fmt);
+   _pSLang_free_slstring (fmt);
 
    SLdo_pop_n (_pSLang_get_run_stack_pointer () - ptr);
 
@@ -1900,7 +1906,7 @@ static void create_delimited_string_cmd (int *nptr)
    str = create_delimited_string (strings + 1, (n - 1), strings[0]);
    /* drop */
    return_error:
-   for (i = 0; i < n; i++) SLang_free_slstring (strings[i]);
+   for (i = 0; i < n; i++) _pSLang_free_slstring (strings[i]);
    SLfree ((char *)strings);
 
    (void) SLang_push_malloced_string (str);   /* NULL Ok */
@@ -2012,7 +2018,7 @@ static void glob_to_regexp (char *glob)
    char *pat, *p;
    char ch;
 
-   len = strlen (glob);
+   len = _pSLstring_bytelen (glob);
    pat = SLmalloc (2*len + 3);
    if (pat == NULL)
      return;
