@@ -204,11 +204,42 @@ static SLang_CStruct_Field_Type TM_Struct [] =
    SLANG_END_CSTRUCT_TABLE
 };
 
+static int validate_tm (struct tm *tms)
+{
+   if ((tms->tm_sec < 0) || (tms->tm_sec > 61)   /* allows for leap-seconds */
+       || (tms->tm_min < 0) || (tms->tm_min > 59)
+       || (tms->tm_hour < 0) || (tms->tm_hour > 23)
+       || (tms->tm_mday < 1) || (tms->tm_mday > 31)
+       || (tms->tm_mon < 0) || (tms->tm_mon > 11)
+       || (tms->tm_wday < 0) || (tms->tm_wday > 6)
+       || (tms->tm_yday < 0) || (tms->tm_yday > 365))
+     {
+	SLang_verror (SL_INVALID_PARM, "Time structure contains invalid values");
+	return -1;
+     }
+   /* The man page specifies no range for is_dst.  Map it to -1,0,1 for safety */
+   if (tms->tm_isdst < -1) tms->tm_isdst = -1;
+   else if (tms->tm_isdst > 1) tms->tm_isdst = 1;
+   return 0;
+}
+
 static int push_tm_struct (struct tm *tms)
 {
    return SLang_push_cstruct ((VOID_STAR) tms, TM_Struct);
 }
 
+static int pop_tm_struct (struct tm *tms)
+{
+   /* The memset is necessary because GLIBC has extra fields that may be
+    * meaningful.  In fact, without it strftime will generate access errors
+    */
+   memset ((char *) tms, 0, sizeof (*tms));
+   if (-1 == SLang_pop_cstruct (tms, TM_Struct))
+     return -1;
+   
+   return validate_tm (tms);
+}
+
 static void localtime_cmd (long *t)
 {
    time_t tt = (time_t) *t;
@@ -367,6 +398,64 @@ static double ftime_cmd (void)
 # endif
 #endif
 
+static void strftime_cmd (void)
+{
+   /* Rather then using some sort of portable version of strftime, which would
+    * miss the locale-specific features, just call the system routine.  However,
+    * it cannot be called blindly because some versions (e.g., the the one from
+    * c.snippets.org) do no input checking, and use code such as
+    *
+    *   static char *day[] = {"Sunday", "Monday", ..., "Saturday"};
+    *   [...]
+    *   switch (*f++)
+    *     {
+    *      case 'A' :
+    *        r = day[t->tm_wday];
+    *        break;
+    *    [...]
+    *
+    * and lead to a SEGV if t->tm_wday is not in the range [0:6].
+    */
+   struct tm tms;
+   char buf[4096];
+   int status;
+   char *fmt;
+
+   if (SLang_Num_Function_Args == 1)
+     {
+	time_t t = time(NULL);
+	tms = *localtime(&t);
+	if (-1 == validate_tm (&tms))
+	  return;
+     }
+   else if (-1 == pop_tm_struct (&tms))
+     return;
+   
+   if (-1 == SLang_pop_slstring (&fmt))
+     return;
+
+   /* Ugh.  The man page says:
+    * 
+    *  The strftime() function returns the number of characters placed in  the
+    *  array  s,  not  including  the  terminating NUL character, provided the
+    *  string, including the terminating NUL, fits.  Otherwise, it returns  0,
+    *  and  the contents of the array is undefined.  (Thus at least since libc
+    *  4.4.4; very old versions of libc, such as libc 4.4.1, would return  max
+    *  if the array was too small.)
+    *  
+    *  Note  that  the  return value 0 does not necessarily indicate an error;
+    *  for example, in many locales %p yields an empty string.
+    * 
+    * Was this too designed by committee? 
+    */
+   status = strftime (buf, sizeof(buf), fmt, &tms);
+   if (status == 0)
+     buf[0] = 0;
+   buf[sizeof(buf)-1] = 0;
+   (void) SLang_push_string (buf);
+   SLang_free_slstring (fmt);
+}
+
 static SLang_Intrin_Fun_Type Time_Funs_Table [] =
 {
    MAKE_INTRINSIC_1("ctime", ctime_cmd, SLANG_STRING_TYPE, SLANG_LONG_TYPE),
@@ -378,7 +467,7 @@ static SLang_Intrin_Fun_Type Time_Funs_Table [] =
 #ifdef HAVE_MKTIME
    MAKE_INTRINSIC_0("mktime", mktime_cmd, SLANG_LONG_TYPE),
 #endif
-
+   MAKE_INTRINSIC_0("strftime", strftime_cmd, SLANG_VOID_TYPE),
 #ifdef HAVE_TIMES
    MAKE_INTRINSIC_0("times", times_cmd, SLANG_VOID_TYPE),
    MAKE_INTRINSIC_0("_tic", _tic_cmd, SLANG_VOID_TYPE),
